#!/usr/bin/python
import os
import shutil

from charmhelpers import fetch
from charmhelpers.core import hookenv, host
from wp_plugin_helpers import WordpressPluginRelation, WordpressPluginOptions


def plugin_path():
    config = hookenv.config()
    hookenv.log(WordpressPluginRelation())
    plugin_dir = WordpressPluginRelation()['wordpress-plugin'][0]['plugin_directory']
    return os.path.join(plugin_dir, config['plugin_name'], '')


def unpack_plugin(service_name):
    path = plugin_path()
    hookenv.log('Fetching plugin code...')
    upstream_code = fetch.install_from_config('code_uri')
    hookenv.log('Installing plugin code in: {}'.format(path))

    # strip off top-level archive directory.
    plugin_files = os.listdir(upstream_code)
    first_archive_file = os.path.join(upstream_code, plugin_files[0])
    if len(plugin_files) == 1 and os.path.isdir(first_archive_file):
        hookenv.log("stripping archive directory {} from path {}".format(
            plugin_files[0], upstream_code))
        upstream_code = first_archive_file

    host.rsync(os.path.join(upstream_code, ''), path)

    index_file = os.path.join(path, 'index.php')
    if not os.path.isfile(index_file):
        # Plugin doesn't install an index.php, so we will
        # add a blank one to prevent directory listing
        with open(index_file, "w") as f:
            f.write("<?php\n// Silence is golden.\n")
            os.chmod(index_file, 0o644)

    hookenv.log("Installed plugin from {} to {}".format(upstream_code,
                                                        path))


def remove_plugin(service_name):
    path = plugin_path()
    hookenv.log('Removing plugin directory: {}'.format(path))
    if os.path.exists(path):
        shutil.rmtree(path)


def wipe_apparmor(service_name):
    config = hookenv.config()
    path = os.path.join('/etc/apparmor.d/wordpress.d', config['plugin_name'])
    if os.path.exists(path):
        os.remove(path)


def notify_plugin_relation(service_name):
    """Notify relations if forced_options changes"""
    config = hookenv.config()
    if not config.changed('forced_options'):
        return
    for rel_id in hookenv.relation_ids('wordpress-plugin'):
        options = WordpressPluginOptions().provide_data()
        hookenv.relation_set(relation_id=rel_id, **options)
