#!/usr/bin/python
import os

from charmhelpers.core.services.base import ServiceManager
from charmhelpers.core.services import helpers
from charmhelpers.core import hookenv

import actions
from wp_plugin_helpers import (WordpressPluginRelation, WordpressPluginName,
        WordpressPluginOptions)


def manage():
    config = hookenv.config()
    manager = ServiceManager([
        {
            'service': 'wordpress-plugin',
            'required_data': [
                helpers.RequiredConfig('plugin_name', 'code_uri'),
                WordpressPluginRelation(),
            ],
            'provided_data': [
                WordpressPluginName(),
                WordpressPluginOptions(),
            ],
            'data_ready': [
                actions.unpack_plugin,
            ],
            'data_lost': [
                actions.remove_plugin,
            ],
            'start': [],
            'stop': [],
        },
        {
            'service': 'wordpress-config',
            'required_data': [
                helpers.RequiredConfig('plugin_name', 'code_uri', 'forced_options'),
            ],
            'data_ready': [
                actions.notify_plugin_relation,
            ],
            'start': [],
            'stop': [],
        },
        {
            'service': 'wordpress-plugin-apparmor',
            'required_data': [
                helpers.RequiredConfig('plugin_name', 'apparmor_rules'),
            ],
            'data_ready': [
                helpers.render_template(source='apparmor.j2',
                                        target=os.path.join(
                                            '/etc/apparmor.d/wordpress.d',
                                            config['plugin_name'])),
            ],
            'data_lost': [
                actions.wipe_apparmor,
            ],
            'start': [],
            'stop': [],
        },
    ])
    manager.manage()

if __name__ == '__main__':
    manage()
