#!/usr/bin/python

from charmhelpers.core import hookenv
from charmhelpers.core.services import helpers


class WordpressPluginRelation(helpers.RelationContext):
    name = 'wordpress-plugin'
    interface = 'wordpress-plugin'
    required_keys = ['plugin_directory']

    def is_ready(self):
        if '{}-relation-departed'.format(self.name) == hookenv.hook_name():
            return False
        else:
            return super(WordpressPluginRelation, self).is_ready()

    def get_data(self):
        """We want relation data in -departed hook."""
        if '{}-relation-departed'.format(self.name) == hookenv.hook_name():
            if not hookenv.relation_ids(self.name):
                return
            ns = self.setdefault(self.name, [])
            ns.append(hookenv.relation_get())
        else:
            super(WordpressPluginRelation, self).get_data()


class WordpressPluginName(helpers.RelationContext):
    name = 'wordpress-plugin'
    interface = 'wordpress-plugin'

    def provide_data(self):
        config = hookenv.config()
        dependencies = set(config['required_plugins'].split())
        rel_active_plugins = hookenv.relation_get('active_plugins') or ''
        installed_plugins = set(rel_active_plugins.split())
        if dependencies - installed_plugins:
            hookenv.log(
                    "{} depends on: {}\nInstalled: {}\nNeeds: {}".format(
                        config['plugin_name'],
                        dependencies,
                        installed_plugins,
                        dependencies - installed_plugins)
                    )
            return {}
        return {'plugin_name': config['plugin_name']}


class WordpressPluginOptions(helpers.RelationContext):
    name = 'wordpress-plugin'
    interface = 'wordpress-plugin'

    def provide_data(self):
        """Decode the dictionary in the yaml, or just pass back an empty one"""
        import yaml
        options = hookenv.config()['forced_options'] or '{}'
        return {'forced_options': yaml.safe_load(options)}
