<?php
/*
 *  Wordpress Launchpad plugin
 *  Copyright (C) 2009-2010 Canonical Ltd.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
Plugin Name: OpenID launchpad
Plugin URI: http://launchpad.net/wordpress-launchpad-integration
Description: Launchpad-specific OpenID changes for Wordpress
Version: 1.0
Author: Canonical ISD
Author URI: http://launchpad.net/~canonical-isd-hackers
*/

define('LAUNCHPAD_PRODUCTION_URL', 'https://login.launchpad.net/');
define('LAUNCHPAD_PRODUCTION_OP', LAUNCHPAD_PRODUCTION_URL.'+openid');
define('LAUNCHPAD_TESTING_URL', 'https://login.demo.launchpad.net/');
define('LAUNCHPAD_TESTING_OP', LAUNCHPAD_TESTING_URL.'+openid');
define('LAUNCHPAD_STAGING_URL', 'https://login.staging.launchpad.net/');
define('LAUNCHPAD_STAGING_OP', LAUNCHPAD_STAGING_URL.'+openid');
define('LAUNCHPAD_SSO_PRODUCTION_URL', 'https://login.ubuntu.com/');
define('LAUNCHPAD_SSO_PRODUCTION_OP', LAUNCHPAD_SSO_PRODUCTION_URL.'+openid');
define('LAUNCHPAD_SSO_STAGING_URL', 'https://login.staging.ubuntu.com/');
define('LAUNCHPAD_SSO_STAGING_OP', LAUNCHPAD_SSO_STAGING_URL.'+openid');

define('LAUNCHPAD_USERPAGE_FORMAT', 'https://launchpad.net/~%s');

add_action('admin_menu', 'openid_launchpad_admin_panels');
add_action('login_head', 'openid_launchpad_wp_login_head');
add_action('wp_authenticate', 'openid_launchpad_wp_authenticate');
add_filter('comments_template', 'openid_launchpad_comment_form', 10);
add_filter('get_comment_author_url', 'openid_launchpad_get_comment_author_url');
add_filter('openid_auth_request_extensions', 'openid_launchpad_check_request', 1, 2);
add_action('wp_login', 'openid_launchpad_wp_login');
add_action('load-profile.php', 'redirect_to_launchpad_profile_page_if_configured');

/**
 *
 */
function openid_launchpad_admin_panels() {
  $hookname = add_options_page(__('Launchpad', 'openid-launchpad'),
                               __('Launchpad', 'openid-launchpad'), 8,
                               'openid-launchpad',
                               'openid_launchpad_page' );
}

function openid_launchpad_page() {
  if (isset($_POST['launchpad_update'])) {
    check_admin_referer('openid-launchpad_update');
    // Set the launchpad server to use
    if (isset($_POST['launchpad_server'])) {
      $srv = openid_launchpad_get_server($_POST['launchpad_server']);
      update_option('openid_launchpad_server', $srv);
    }
    // Should local user data be kept in sync with launchpad each login
    update_option('openid_launchpad_sync', isset($_POST['launchpad_sync']) ? true : false);

    // Should users be redirected to their launchpad profile page if they have one?
    update_option('openid_launchpad_profile', isset($_POST['launchpad_profile']) ? true : false);
  }

  ?>
  <div class="wrap">
    <form method="post">
      <h2><?php _e('Launchpad Options', 'openid-launchpad') ?></h2>
      <table class="form-table optiontable editform" cellspacing="2" cellpadding="5" width="100%">
        <tr valign="top">
          <th scope="row"><?php _e('Launchpad Server', 'openid-launchpad') ?></th>
          <td>
            <p><select name="launchpad_server"><?php
            $options = array(
              'PRODUCTION'     => __('Production', 'openid-launchpad'),
              'TESTING'        => __('Testing', 'openid-launchpad'),
              'STAGING'        => __('Staging', 'openid-launchpad'),
              'SSO_PRODUCTION' => __('Ubuntu SSO production', 'openid-launchpad'),
              'SSO_STAGING'    => __('Ubuntu SSO staging', 'openid-launchpad'),
            );
            foreach ($options as $key => $label) {
              $selected = openid_launchpad_get_server();
              if ($selected == constant('LAUNCHPAD_' . $key . '_URL')) {
                $selected = ' selected="selected"';
              } else {
                $selected = '';
              }
              printf('<option value="%s"%s>%s</option>', $key,
                     $selected, htmlentities($label));
            }
            ?></select></p>
          </td>
        </tr>
        <tr valign="top">
          <th scope="row"><?php _e('Sync user data', 'openid-launchpad') ?></th>
          <td>
            <p><input type="checkbox" name="launchpad_sync" id="launchpad_sync" value="1"<?php
            if (get_option('openid_launchpad_sync')) {
              echo ' checked="checked"';
            }
            ?> /> <?php _e('Should wordpress attempt to sync user data such as nickname, name, email with each login?', 'openid-launchpad') ?></p>
          </td>
        </tr>
        <tr valign="top">
          <th scope="row"><?php _e('Redirect to launchpad profile', 'openid-launchpad') ?></th>
          <td>
            <p><input type="checkbox" name="launchpad_profile" id="launchpad_profile" value="1"<?php
            if (get_option('openid_launchpad_profile')) {
              echo ' checked="checked"';
            }
            ?> /> <?php _e('Should wordpress redirect the user to their launchpad profile page if they have one?', 'openid-launchpad') ?></p>
          </td>
        </tr>
      </table>
      <?php wp_nonce_field('openid-launchpad_update'); ?>
      <p class="submit"><input type="submit" name="launchpad_update" value="<?php _e('Update Options') ?> &raquo;" /></p>
    </form>
  </div>
  <?php
}

/**
 *
 */
function openid_launchpad_get_server($type = null) {
  if (empty($type)) {
    $srv = get_option('openid_launchpad_server');
    if (!empty($srv)) {
      return $srv;
    }
  } else {
    $const = 'LAUNCHPAD_' . $type . '_URL';
    if (defined($const)) {
      return constant($const);
    }
  }
  return constant('LAUNCHPAD_PRODUCTION_URL');
}

/**
 * Redirect to the custom launchpad login page and exit
 */
function openid_launchpad_wp_login_head() {
    /* Only include the custom login form for plain vanilla sign in */
    if (!isset($_REQUEST['action']) &&
          (!isset($_GET['checkemail']) ||
           !in_array( $_GET['checkemail'], array('confirm', 'newpass')))) {
        include 'launchpad_login.php';
        exit();
    }
}

/**
 * Handle the authentication request for launchpad sign-in
 *
 * @param string $credentials username and password provided in login
 *                            form
 */
function openid_launchpad_wp_authenticate(&$credentials) {
  if (isset($_REQUEST['launchpad'])) {
    openid_start_login(openid_launchpad_get_server(), 'login',
                      $_REQUEST['redirect_to']);
  }
}

/**
 * Display an alternative comments form for better launchpad
 * integration.
 *
 * Looks in the following places (and order):
 *   1. {$current_theme_dir}/launchpad_comments.php
 *   2. {$openid-launchpad_plugin_dir}/launchpad_comments.php
 *   3. The normal comments form
 *
 * @param string $file
 * @return string
 */
function openid_launchpad_comment_form($file) {
  if (file_exists(dirname($file).'/launchpad_comments.php')) {
    return dirname($file).'/launchpad_comments.php';
  } elseif (file_exists(dirname(__FILE__).'/launchpad_comments.php')) {
    return dirname(__FILE__).'/launchpad_comments.php';
  } else {
    return $file;
  }
}

/**
 * Display the user's launchpad user page in comments instead of their
 * launchpad id as the latter doesn't contain any useful data for
 * normal people.
 *
 * @param string $url
 * @return string
 */
function openid_launchpad_get_comment_author_url($url) {
  if (is_comment_openid()) {
    $user = get_userdata_by_various($url);
    if ($user) {
      return sprintf(LAUNCHPAD_USERPAGE_FORMAT, $data['nickname']);
    }
  }
  return $url;
}

/**
 * Sync the user's details with launchpad
 */
function openid_launchpad_wp_login() {
  if (isset($_GET['openid_mode']) && get_option('openid_launchpad_sync')) {
    $user = wp_get_current_user();
    $sreg = openid_get_user_data_sreg(array(), null);
    $new  = array();
    $fields = array('nickname:user_login', 'user_email', 'nickname',
                    'user_nicename', 'display_name', 'first_name',
                    'last_name');
    foreach ($fields as $key) {
      if (strpos($key, ':') !== false) {
        list($key, $val) = explode(':', $key, 2);
      } else {
        $val = $key;
      }
      if (!empty($sreg[$key]) && $sreg[$key] != $user->$val) {
        $new[$val] = $sreg[$key];
      }
    }
    if (sizeof($new) > 0) {
      $new['ID'] = $user->ID;
      require_once ABSPATH . WPINC .  '/registration.php';
      wp_update_user($new);
    }
  }
}

function redirect_to_launchpad_profile_page_if_configured() {
  $user = wp_get_current_user();
  if (should_redirect_to_launchpad_profile_page() && user_registered_with_launchpad($user)) {
    header("Location: http://launchpad.net/~{$user->user_login}");
    exit();
  }
}

function should_redirect_to_launchpad_profile_page() {
  return (boolean) get_option('openid_launchpad_profile');
}

function user_registered_with_launchpad($user) {
  $all_urls = get_user_openids($user->ID);
  foreach ($all_urls as $url) {
    $parts = parse_url($url);
    if (array_key_exists('host', $parts) && eregi('\.launchpad\.net$', $parts['host'])) {
      return true;
    }
  }
  return false;
}

/**
 * Filter to check if the endpoint server in the current request is the one set
 *
 * @param array $extensions
 * @param object $auth_request
 */
function openid_launchpad_check_request($extensions, $auth_request) {
  if (!is_current_launchpad_endpoint($auth_request->endpoint->server_url)) {
    exit('Permission denied: You cannot login using that server');
  }
}

/**
 * Check the given endpoint to see if it matches the currently selected one
 *
 * @param string $endpoint
 * @return boolean
 */
function is_current_launchpad_endpoint($endpoint) {
  $current = openid_launchpad_get_server();
  foreach (array('PRODUCTION', 'TESTING', 'STAGING', 'SSO_PRODUCTION', 'SSO_STAGING') as $server) {
    if (constant("LAUNCHPAD_{$server}_URL") == $current) {
      return constant("LAUNCHPAD_{$server}_OP") == $endpoint;
    }
  }
  return false;
}

?>
